/*=========================================================================

 Program:   BlueBerry Platform
 Language:  C++
 Date:      $Date: 2010-01-16 19:57:43 +0100 (Sat, 16 Jan 2010) $
 Version:   $Revision: 21070 $

 Copyright (c) German Cancer Research Center, Division of Medical and
 Biological Informatics. All rights reserved.
 See MITKCopyright.txt or http://www.mitk.org/copyright.html for details.

 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/


#ifndef BERRYIELEMENTREFERENCE_H_
#define BERRYIELEMENTREFERENCE_H_

#include <osgi/framework/Object.h>
#include <osgi/framework/Macros.h>

#include "../berryUiDll.h"

#include <map>

namespace berry {

class UIElement;

/**
 * the ICommandService will return a reference for all callbacks that are
 * registered. This reference can be used to unregister the specific callback.
 * <p>
 * Similar in functionality to an IHandlerActivation.  This interface should
 * not be implemented or extended by clients.
 * </p>
 *
 * @since 3.3
 */
struct BERRY_UI IElementReference : public Object {

  osgiInterfaceMacro(berry::IElementReference)

  /**
   * The command id that this callback was registered against.
   *
   * @return The command id. Will not be <code>null</code>.
   */
  virtual std::string GetCommandId() const = 0;

  /**
   * The callback that was registered.
   *
   * @return Adapts to provide appropriate user feedback. Will not be
   *         <code>null</code>.
   */
  virtual SmartPointer<UIElement> GetElement() const = 0;

  /**
   * Parameters that help scope this callback registration. For example, it
   * can include parameters from the ParameterizedCommand.
   *
   * @return scoping parameters. Will not be <code>null</code>.
   */
  virtual std::map<std::string, Object::Pointer> GetParameters() const = 0;
};

}

#endif /* BERRYIELEMENTREFERENCE_H_ */
